use anyhow::Result;

use std::str::FromStr;

#[derive(Debug)]
enum Direction {
    Forward(u64),
    Down(u64),
    Up(u64),
}

impl FromStr for Direction {
    type Err = anyhow::Error;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        if let Some((direction, distance)) = s.split_once(" ") {
            let distance = distance.parse()?;

            Ok(match direction {
                "forward" => Direction::Forward(distance),
                "down" => Direction::Down(distance),
                "up" => Direction::Up(distance),
                _ => panic!("not a command"),
            })
        } else {
            Err(anyhow::format_err!("could not split direction"))
        }
    }
}

fn part_1() -> Result<u64> {
    let cmds = aoc::read_one_per_line::<Direction>("./data/02.directions")?;
    //                    horizontal, depth
    let loc = cmds.iter().fold((0, 0), |mut loc, dir| match dir {
        Direction::Forward(dis) => {
            loc.0 += dis;
            loc
        }
        Direction::Down(dis) => {
            loc.1 += dis;
            loc
        }
        Direction::Up(dis) => {
            loc.1 -= dis;
            loc
        }
    });
    Ok(loc.0 * loc.1)
}

fn part_2() -> Result<u64> {
    let cmds = aoc::read_one_per_line::<Direction>("./data/02.directions")?;
    //                    horizontal, depth, aim
    let loc = cmds.iter().fold((0, 0, 0), |mut loc, dir| match dir {
        Direction::Forward(dis) => {
            loc.0 += dis;
            loc.1 += dis * loc.2;
            loc
        }
        Direction::Down(dis) => {
            loc.2 += dis;
            loc
        }
        Direction::Up(dis) => {
            loc.2 -= dis;
            loc
        }
    });
    Ok(loc.0 * loc.1)
}

fn main() -> Result<()> {
    println!("Part 1: {}", part_1()?);
    println!("Part 2: {}", part_2()?);

    Ok(())
}
